@@ -182,18 +182,19 @@ If no value is provided, the whole json will be compared.
                 ),
         )
         .arg(
-            Arg::with_name("raw-json")
-                .long("raw-json")
-                .default_value("socorro")
+            Arg::with_name("evil-json")
+                .long("evil-json")
+                .default_value("default")
                 .takes_value(true)
                 .long_help(
                     "An input JSON file with the extra information.
 
 Possible values:
-* 'socorro' (default) - Get the 'raw' JSON from socorro 
+* 'default' - 'socorro' if using socorro (--crash-id), 'none' otherwise
+* 'none' - Do not use an evil-json
+* 'socorro' - Get the 'raw' JSON from socorro 
   (API token may require 'View Personal Identifiable Information') 
-* 'none' - Do not use a raw-json
-* <anything else> - assumed to be a path to a local raw-json file
+* <anything else> - assumed to be a path to a local evil-json file
 
 This is a gross hack for some legacy side-channel information that mozilla uses. It will \
 hopefully be phased out and deprecated in favour of just using custom streams in the \
@@ -332,23 +333,29 @@ so this option is a confusing trap.
             Arg::with_name("api-token")
                 .long("api-token")
                 .takes_value(true)
-                .required(true)
                 .long_help(
                     "A socorro api token to use for getting minidumps
 
 Required permissions:
 
 * 'View Raw Dumps' (hard required)
-* 'View Personal Identifiable Information' (to read some entries in raw-json)
+* 'View Personal Identifiable Information' (to read some entries in raw-json/evil-json)
 
 See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
                 ),
         )
+        .arg(
+            Arg::with_name("minidump")
+                .long("minidump")
+                .takes_value(true)
+                .long_help(
+                    "A local minidump to analyze (instead of using --api-token and --crash-id) \n\n\n",
+                ),
+        )
         .arg(
             Arg::with_name("crash-id")
                 .long("crash-id")
                 .takes_value(true)
-                .required(true)
                 .long_help(
                     "The socorro crash id to analyze.\n\n",
                 ),
@@ -486,11 +493,59 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
         .map(|v| v.map(String::from).collect::<Vec<_>>())
         .unwrap_or_else(Vec::new);
 
-    let raw_json_arg = matches.value_of_os("raw-json").expect("Missing raw json");
+    let evil_json_arg = matches.value_of_os("evil-json").expect("Missing evil json");
 
-    let api_token = matches.value_of("api-token").expect("Missing API token");
+    let api_token = matches.value_of("api-token");
+    let crash_id = matches.value_of("crash-id");
+    let local_minidump = matches.value_of("minidump").map(|path| PathBuf::from(path));
 
-    let crash_id = matches.value_of("crash-id").expect("Missing crash-id");
+    let using_socorro = api_token.is_some() && crash_id.is_some();
+    let trying_to_use_socorro = api_token.is_some() || crash_id.is_some();
+    let using_local_minidump = local_minidump.is_some();
+
+    if using_local_minidump && trying_to_use_socorro {
+        let err = "--minidump is incompatible with --api-token and --crash-id".to_string();
+        eprintln!("ERROR: {}\n", err);
+        return Err(err.into());
+    }
+
+    if !using_socorro && trying_to_use_socorro {
+        let err = "--api-token and --crash-id must be used together".to_string();
+        eprintln!("ERROR: {}\n", err);
+        return Err(err.into());
+    }
+
+    let _local_minidump_str;
+    let crash_id = if let Some(crash_id) = crash_id {
+        // using socorro
+        crash_id
+    } else {
+        // using local
+        _local_minidump_str = local_minidump
+            .as_ref()
+            .unwrap()
+            .file_name()
+            .unwrap()
+            .to_string_lossy()
+            .into_owned();
+        &_local_minidump_str
+    };
+
+    let evil_json_arg = if evil_json_arg == OsStr::new("default") {
+        if using_local_minidump {
+            OsStr::new("none")
+        } else {
+            OsStr::new("socorro")
+        }
+    } else {
+        evil_json_arg
+    };
+
+    if !using_socorro && evil_json_arg == OsStr::new("socorro") {
+        let err = "--evil-json=socorro requires socorro (--crash-id)".to_string();
+        eprintln!("ERROR: {}\n", err);
+        return Err(err.into());
+    }
 
     let default_ignored_fields = [
         // deprecated
@@ -579,7 +634,7 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
     // output paths to report:
     // * minidump (the minidump to analyze)
     // * socc_output (socorro processed crash)
-    // * raw_json ("raw" json file)
+    // * evil_json ("extra" json file AKA evil-json)
     // * local_json_output (local minidump-stackwalk --json output)
     // * local_human_output (local minidump-stackwalk --human output)
     // * local_logs (local minidump-stackwalk log output)
@@ -593,45 +648,61 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
     let mut local_logs = dump_cache.join(&crash_id);
     local_logs.set_extension("log.txt");
 
-    // Download the minidump
-    let minidump_req = reqwest::Client::new()
-        .get("https://crash-stats.mozilla.org/api/RawCrash/")
-        .header("Auth-Token", api_token)
-        .query(&[
-            ("crash_id", crash_id),
-            ("format", "raw"),
-            ("name", "upload_file_minidump"),
-        ]);
+    let minidump = if let Some(api_token) = &api_token {
+        let mut minidump_dest = dump_cache.join(&crash_id);
+        minidump_dest.set_extension("dmp");
 
-    let mut minidump = dump_cache.join(&crash_id);
-    minidump.set_extension("dmp");
+        // Download the minidump
+        let minidump_req = reqwest::Client::new()
+            .get("https://crash-stats.mozilla.org/api/RawCrash/")
+            .header("Auth-Token", *api_token)
+            .query(&[
+                ("crash_id", crash_id),
+                ("format", "raw"),
+                ("name", "upload_file_minidump"),
+            ]);
 
-    fetch(f, minidump_req, &minidump).await?;
+        fetch(f, minidump_req, &minidump_dest).await?;
 
-    // Download the json socorro has for the minidump
-    let socc_output_req = reqwest::Client::new()
-        .get("https://crash-stats.mozilla.org/api/ProcessedCrash/")
-        .header("Auth-Token", api_token)
-        .query(&[("crash_id", crash_id)]);
-    let mut socc_output = dump_cache.join(&crash_id);
-    socc_output.set_extension("json");
+        minidump_dest
+    } else {
+        local_minidump.unwrap()
+    };
 
-    let socc_json = fetch(f, socc_output_req, &socc_output).await?;
-    let socc_json: serde_json::Value = serde_json::from_reader(BufReader::new(socc_json)).unwrap();
-    let socc_json = socc_json.get("json_dump").unwrap();
+    let mut socc_output = None;
+    let mut socc_json = None;
+    let tmp_socc_json2: serde_json::Value;
+
+    if let Some(api_token) = &api_token {
+        // Download the json socorro has for the minidump
+        let socc_output_req = reqwest::Client::new()
+            .get("https://crash-stats.mozilla.org/api/ProcessedCrash/")
+            .header("Auth-Token", *api_token)
+            .query(&[("crash_id", crash_id)]);
+        let mut tmp_socc_output = dump_cache.join(&crash_id);
+        tmp_socc_output.set_extension("json");
+
+        if !skip_diff {
+            let tmp_socc_json1 = fetch(f, socc_output_req, &tmp_socc_output).await?;
+            tmp_socc_json2 = serde_json::from_reader(BufReader::new(tmp_socc_json1)).unwrap();
+
+            socc_json = tmp_socc_json2.get("json_dump");
+        }
+        socc_output = Some(tmp_socc_output);
+    }
 
     // Get the "raw" json file with things like certificate info
-    let raw_json = if raw_json_arg == OsStr::new("none") {
+    let evil_json = if evil_json_arg == OsStr::new("none") {
         // Don't use raw json
         None
-    } else if raw_json_arg == OsStr::new("socorro") {
+    } else if evil_json_arg == OsStr::new("socorro") {
         // Download the raw json from socorro
         let extra_json_req = reqwest::Client::new()
             .get("https://crash-stats.mozilla.org/api/RawCrash/")
-            .header("Auth-Token", api_token)
+            .header("Auth-Token", api_token.unwrap())
             .query(&[("crash_id", crash_id)]);
         let mut extra_json_path = dump_cache.join(&crash_id);
-        extra_json_path.set_extension("raw.json");
+        extra_json_path.set_extension("evil.json");
 
         // Just need it on disk for rust-minidump, don't actually need the value here.
         let _extra_json = fetch(f, extra_json_req, &extra_json_path).await?;
@@ -639,7 +710,7 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
         Some(extra_json_path)
     } else {
         // Assume --raw-json is a local path
-        Some(PathBuf::from(raw_json_arg))
+        Some(PathBuf::from(evil_json_arg))
     };
 
     //
@@ -713,6 +784,21 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
         install_bin(f, &install_tmp, "minidump-stackwalk", ">= 0.9.6")?
     };
 
+    let mdsw_version = {
+        let output = Command::new(&minidump_stackwalk_bin)
+            .arg("-V")
+            .output()?
+            .stdout;
+        let output_str = std::str::from_utf8(&output)
+            .expect("minidump-stackwalk -V wasn't utf8??")
+            .trim();
+        let (name, version) = output_str
+            .split_once(' ')
+            .expect("Couldn't parse minidump-stackwalk version");
+        assert_eq!(name, "minidump-stackwalk");
+        semver::Version::parse(version).expect("Could not parse minidump-stackwalk version")
+    };
+
     //
     //
     //
@@ -738,9 +824,13 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
     let mut command_temp = Command::new(&minidump_stackwalk_bin);
     let mut command = &mut command_temp;
 
-    if let Some(raw_json) = &raw_json {
-        let mut arg = OsString::from("--raw-json=");
-        arg.push(&raw_json);
+    if let Some(evil_json) = &evil_json {
+        let mut arg = if mdsw_version >= semver::Version::parse("0.10.0").unwrap() {
+            OsString::from("--evil-json=")
+        } else {
+            OsString::from("--raw-json=")
+        };
+        arg.push(&evil_json);
         command = command.arg(arg);
     }
 
@@ -1002,7 +1092,7 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
     if statuses.iter().all(|s| s.success()) {
         writeln!(f, "all done!\n")?;
 
-        if !skip_diff {
+        if let Some(socc_json) = &socc_json {
             // Note, only the results of the last run will be used
             // (we assume they're all equivalent).
             writeln!(f, "comparing json...")?;
@@ -1158,24 +1248,30 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
 
     writeln!(f)?;
     writeln!(f, "Output Files: ")?;
-    writeln!(f, "  * (download) Minidump: {}", minidump.display())?;
-    writeln!(
-        f,
-        "  * (download) Socorro Processed Crash: {}",
-        socc_output.display()
-    )?;
-    if let Some(raw_json) = &raw_json {
-        writeln!(f, "  * (download) Raw JSON: {}", raw_json.display())?;
+    if using_local_minidump {
+        writeln!(f, "  * Minidump: {}", minidump.display())?;
+    } else {
+        writeln!(f, "  * (download) Minidump: {}", minidump.display())?;
+    }
+    if let Some(socc_output) = &socc_output {
+        writeln!(
+            f,
+            "  * (download) Socorro Processed Crash: {}",
+            socc_output.display()
+        )?;
+    }
+    if let Some(evil_json) = &evil_json {
+        writeln!(f, "  * (download) Evil JSON: {}", evil_json.display())?;
     }
     writeln!(
         f,
-        "  * Local minidump-stackwalk json Output: {}",
+        "  * Local minidump-stackwalk --json Output: {}",
         local_json_output.display()
     )?;
     if cyborg {
         writeln!(
             f,
-            "  * Local minidump-stackwalk human Output: {}",
+            "  * Local minidump-stackwalk --human Output: {}",
             local_human_output.display()
         )?;
     }
